use clap::Parser;
use log::{debug, info, LevelFilter};

use ambient_driver::{
    config::{Args, EffectiveConfig},
    run::run,
};

fn main() {
    if let Err(e) = fallible_main() {
        eprintln!("ERROR: {}", e);
        let mut source = e.source();
        while let Some(src) = source {
            eprintln!("caused by: {}", src);
            source = src.source();
        }
        std::process::exit(1);
    }
}

fn fallible_main() -> anyhow::Result<()> {
    setup_env_logger("AMBIENT_LOG", LevelFilter::Info);
    info!("ambient-driver starts");
    let args = Args::parse();
    let config = EffectiveConfig::try_from(&args)?;
    debug!("{:#?}", config);
    run(&config)?;
    info!("ambient-driver ends successfully");
    Ok(())
}

fn setup_env_logger(var_name: &str, level_filter: LevelFilter) {
    if std::env::var(var_name).is_ok() {
        env_logger::init_from_env(var_name);
    } else {
        env_logger::builder().filter_level(level_filter).init();
    }
}
